using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.XPath;

namespace BennettAdelson.Samples.FXCopRuleExplorer
{
    internal class RuleXMLUtilities
    {
        private XmlDocument xd;
        private XPathNavigator xn;
        private object syncObj = new object();

        public RuleXMLUtilities(XmlDocument xd)
        {
            this.xd = xd;
            this.xn = xd.CreateNavigator();
        }

        #region Properties
        private int m_NumRules;

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode", Justification = "Data bound to PropertyGrid")]
        public int NumberOfRules
        {
            get
            {
                if (m_NumRules == 0)
                {
                    m_NumRules = (int)(double)xn.Evaluate(@"count(//Rule)");
                }
                return m_NumRules;
            }
        }

        private double m_AvgCertainty;
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode",Justification="Data bound to PropertyGrid")]
        public double AverageCertainty
        {
            get
            {
                if (m_AvgCertainty == 0)
                {
                    m_AvgCertainty =
                        (double)xn.Evaluate(@"sum(//Rule/MessageLevel/@Certainty) div count(//Rule/MessageLevel/@Certainty)");                   
                }

                return m_AvgCertainty;
            }
        }

        private int m_NumFound;
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode", Justification = "Data bound to PropertyGrid")]
        public int NumFound
        {
            get { return m_NumFound; }
            set { m_NumFound = value; }
        }
        #endregion

        public XmlDocument GetCurrentDocument() { return (XmlDocument)xd.Clone(); }

        public XmlDocument FilterByRuleName(string containsName)
        {
            lock (syncObj)
            {
                int hits = 0;
                XmlDocument rd = new XmlDocument();
                string comment = string.Format("Rules with Names or Descriptions containing '{0}'", containsName);
                rd.AppendChild(rd.CreateComment(comment));

                rd.AppendChild(rd.CreateElement("Rules"));

                string target = containsName.ToLower();
                string lastFound = "?";

                foreach (XmlNode n in xd.SelectNodes(@"//Rule/Name | //Rule/Description"))
                {
                    XmlElement e = n as XmlElement;
                    if (e.InnerText.ToLower().Contains(target))
                    {
                        if (n.ParentNode.Attributes["TypeName"].Value != lastFound)
                        {
                            rd.DocumentElement.AppendChild(rd.ImportNode(n.ParentNode, true));
                            hits++;
                            lastFound = n.ParentNode.Attributes["TypeName"].Value;
                        }
                    }
                }

                this.NumFound = hits;
                rd.FirstChild.Value += " (" + hits.ToString() + " rules)";
                return rd;
            }
        }

    }
}
