using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Cci;
using Microsoft.FxCop.Sdk.Introspection;
using Microsoft.FxCop.Sdk;

namespace BennettAdelson.SampleFXCopRules
{
    internal sealed class PreventUncontrolledRecursion : BaseIntrospectionRule
    {
        //public PreventUncontrolledRecursion() : this("PreventUncontrolledRecursion") { }

        //public PreventUncontrolledRecursion(string name)
        //    :
        //    base(name,
        //            "BennettAdelson.SampleFXCopRules.Rules",
        //            typeof(PreventUncontrolledRecursion).Assembly)
        //{
        //    ;
        //}

        internal const int MAX_SHORT_VARS_PER_METHOD = 5;
        internal const int MIN_VARNAME_LENGTH = 3;

        public PreventUncontrolledRecursion()
			:base(
                typeof(PreventUncontrolledRecursion).Name,
                typeof(PreventUncontrolledRecursion).Namespace + ".Rules",
                typeof(PreventUncontrolledRecursion).Assembly
			)
		{}


        public override ProblemCollection Check(Microsoft.Cci.Member member)
        {
            Method method = member as Method;

            if (method == null)
            {
                // Not a Method
                return null;
            }

            LocalList locals = null;
            if (method.Instructions.Length > 0)
            {
                locals = method.Instructions[0].Value as LocalList;
            }

            int shortVarCount = 0;

            if (locals != null)
            {
                for (int li = 0; li < locals.Length; li++)
                {
                    if (locals[li].Name.Name.Length < MIN_VARNAME_LENGTH)
                    {
                        shortVarCount++;
                    }
                }
            }

            if (shortVarCount > MAX_SHORT_VARS_PER_METHOD)
            {
                Problems.Add(new Problem(GetResolution(method.Name.Name)));
            }

            #region Other Trys
            //See if this method calls itself...
            //if (IsMethodCallAvailable(method.Name.Name, method))
            //{
            //    Problems.Add(new Problem(GetResolution(method.Name.Name)));
            //}

            //int instNum = 0;

            //foreach (Instruction inst in method.Instructions)
            //{
            //    if (inst.OpCode == OpCode.Call || inst.OpCode == OpCode.Calli || inst.OpCode == OpCode.Callvirt)
            //    {
            //        Method calledMethod = inst.Value as Method;
            //        if (calledMethod.Name.Name == method.Name.Name)
            //        {
            //            //this Method Calls itself, now see if this call is "controlled"...
                        
            //            //walk up the the instruction list to find a branch
            //            Instruction curInst = inst;
            //            Instruction PrevBranch;

            //            while (instNum > 0)
            //            {
            //                curInst = method.Instructions[--instNum];
            //                switch (curInst.OpCode)
            //                {
            //                    case OpCode.Bne_Un_S1:
            //                    case OpCode.Bne_Un:
            //                        break;
            //                    default:
            //                        break;
            //                }

            //                switch (curInst.OpCode)
            //                {
            //                    case OpCode.Br:
            //                    case OpCode.Beq:
            //                    case OpCode.Beq_S1:
            //                    case OpCode.Bge:
            //                    case OpCode.Bge_S1:
            //                    case OpCode.Bge_Un:
            //                    case OpCode.Bge_Un_S1:
            //                    case OpCode.Bgt:
            //                    case OpCode.Bgt_Un_S1:
            //                    case OpCode.Ble:
            //                    case OpCode.Ble_S1:
            //                    case OpCode.Ble_Un:
            //                    case OpCode.Ble_Un_S1:
            //                    case OpCode.Blt:
            //                    case OpCode.Blt_S1:
            //                    case OpCode.Blt_Un:
            //                    case OpCode.Blt_Un_S1:
            //                    case OpCode.Bne_Un:
            //                    case OpCode.Bne_Un_S1:
            //                    case OpCode.Br_S:
            //                    case OpCode.Brfalse:
            //                    case OpCode.Brfalse_S:
            //                    case OpCode.Brtrue:
            //                    case OpCode.Brtrue_S:
            //                        PrevBranch = curInst;
            //                        break;
            //                    default:
            //                        break;
            //                }
            //            }
            //        }
            //    }

            //    instNum++;
            //}

            //foreach (Instruction inst in method.Instructions)
            //{
            //    System.Diagnostics.Debug.WriteLine("JDF-INST:" + inst.OpCode);
                
            //}

            //if (method.Body != null)
            //{
            //    if (method.Body.Statements != null)
            //    {
            //        foreach (Statement stmt in method.Body.Statements)
            //        {
            //            if (stmt != null)
            //                System.Diagnostics.Debug.WriteLine("JDF-STMT:" + stmt.UniqueKey);
            //            else
            //                System.Diagnostics.Debug.WriteLine("JDF-STMT:null!");
            //        }
            //    }
            //}

            //if (method.References != null)
            //{
            //    foreach (Node reference in method.References)
            //    {
            //        System.Diagnostics.Debug.WriteLine("JDF-REF:" + reference.SourceContext.SourceText);
            //    }

            //}

            //return base.Check(member);
            #endregion

            return Problems;
        }

        private bool IsMethodCallAvailable(string methodName, Method method)
        {
            foreach (Instruction instruction in method.Instructions)
            {
                if (instruction.OpCode == OpCode.Call || instruction.OpCode == OpCode.Callvirt)
                {
                    Method calledMethod = instruction.Value as Method;
                    if (calledMethod.Name.Name == methodName)
                        return true;
                }
            }
            return false;
        }

        //private bool IsIfStatementAvailable(Method method)
        //{
        //    foreach (Instruction instruction in method.Instructions)
        //    {
        //        if (instruction.OpCode == OpCode.Br
        //    }
        //}


    }
}
