using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Cci;
using Microsoft.FxCop.Sdk.Introspection;
using Microsoft.FxCop.Sdk;

namespace BennettAdelson.SampleFXCopRules
{
    internal sealed class ConsiderMoreDescriptiveLocalVariableNames : BaseIntrospectionRule
    {
        internal const int MAX_SHORT_VARS_PER_METHOD = 5;
        internal const int MIN_VARNAME_LENGTH = 3;

        //tell Framework The Exact Embedded Resource Name of Rule XML
        public ConsiderMoreDescriptiveLocalVariableNames()
			:base(
                typeof(ConsiderMoreDescriptiveLocalVariableNames).Name,
                typeof(ConsiderMoreDescriptiveLocalVariableNames).Namespace + ".Rules",
                typeof(ConsiderMoreDescriptiveLocalVariableNames).Assembly
            ) { ; }


        public override ProblemCollection Check(Microsoft.Cci.Member member)
        {
            Method method = member as Method;

            if (method == null)
            {
                // Not a Method
                return null;
            }

            // Attempt to find any locally declared variables
            LocalList locals = null;
            if (method.Instructions.Length > 0)
            {
                locals = method.Instructions[0].Value as LocalList;
            }

            // Maintain a count of Short Local Variable Names for this Method
            int shortVarCount = 0;
            List<string> shortVarList = new List<string>();

            // Count up the Number of Short Variable Names in this method
            if (locals != null)
            {
                for (int li = 0; li < locals.Length; li++)
                {
                    if (!RuleUtilities.IsCompilerGenerated(locals[li]))
                    {
                        if (locals[li].Name.Name.Length < MIN_VARNAME_LENGTH)
                        {
                            shortVarCount++;
                            shortVarList.Add(locals[li].Name.Name);
                        }
                    }
                }
            }

            // Raise a Policy Violation if too many short variable names were found in this method
            if (shortVarCount > MAX_SHORT_VARS_PER_METHOD)
            {
                // The method '{0}' declares the following {1} variables: '{2}'.  For maintainability, consider using longer, more descriptive variable names such that no more than {3} variables that are less than {4} characters in length existing within a single method.
                Problems.Add(new Problem(GetResolution(method.Name.Name, 
                                                        shortVarCount.ToString(), 
                                                        string.Join(",",shortVarList.ToArray()),
                                                        MAX_SHORT_VARS_PER_METHOD.ToString(), 
                                                        MIN_VARNAME_LENGTH.ToString())));
            }

            return Problems;
        }
    }
}
