using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Net;
using System.Threading;
using System.Text;
using System.Windows.Forms;

using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.Proxy;

namespace WarehouseUpdates
{
    public partial class MainForm : Form
    {
        private TeamFoundationServer m_Server;

        private WarehouseControllerService.ControllerService svc =
                new WarehouseUpdates.WarehouseControllerService.ControllerService();

        private readonly string settingName = "RunIntervalSeconds";

        public MainForm()
        {
            InitializeComponent();

            //use configured Default Server
            m_Server = TeamFoundationServerFactory.GetServer(WarehouseUpdates.Properties.Settings.Default.PreferredServer);
            initializeCurrentServer();
        }

        private void btnSelectServer_Click(object sender, EventArgs e)
        {
            //prompt for server
            DomainProjectPicker dpp = new DomainProjectPicker(DomainProjectPickerMode.None);
            DialogResult dr = dpp.ShowDialog(this);
            if (dr == DialogResult.OK)
            {
                //use User Selection
                m_Server = dpp.SelectedServer;
            }

            initializeCurrentServer();
        }

        private void initializeCurrentServer()
        {
            lblServer.Text = m_Server.Name;

            UriBuilder url = new UriBuilder(svc.Url);
            url.Host = m_Server.Name;
            svc.Url = url.ToString();

            svc.Credentials = CredentialCache.DefaultCredentials;

            updateCurrentInterval();

            btnChangeInterval.Enabled = true;
            mskUpdateInterval.Enabled = true;
            btnUpdateNow.Enabled = true;
        }

        private void btnChangeInterval_Click(object sender, EventArgs e)
        {
            int newInterval;
            if (int.TryParse(mskUpdateInterval.Text, out newInterval))
            {
                svc.ChangeSetting(settingName, newInterval.ToString());
                updateCurrentInterval();
            }
        }

        private void updateCurrentInterval()
        {
            int currentInterval = svc.GetNextInterval(0);
            mskUpdateInterval.Text = currentInterval.ToString();
            lblUpdateStatus.Text = svc.GetWarehouseStatus().ToString();
        }

        private void btnUpdateNow_Click(object sender, EventArgs e)
        {
            try
            {
                //set UI
                prgBarUpdate.Value = 0;
                btnUpdateNow.Enabled = false;
                grpServerSettings.Enabled = false;

                //start update
                if (!svc.Run()) throw new ApplicationException("Failed to start update.");

                //start background timer thread to poll status
                timer1.Enabled = true;

                //perform first check
                checkWarehouseUpdateStatus();
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.ToString());

                btnUpdateNow.Enabled = true;
                grpServerSettings.Enabled = true;
            }            
        }
       
        private void timer1_Tick(object sender, EventArgs e)
        {
            prgBarUpdate.Value = (prgBarUpdate.Value + 1) % 10;
            if (prgBarUpdate.Value == 0)
                checkWarehouseUpdateStatus();
        }

        private void checkWarehouseUpdateStatus()
        {
            WarehouseControllerService.WarehouseStatus 
                currentStatus = svc.GetWarehouseStatus();

            if (currentStatus != WarehouseUpdates.WarehouseControllerService.WarehouseStatus.Idle)
            {
                lblUpdateStatus.Text = currentStatus.ToString();
                lblUpdateStatus.Refresh();
            }
            else
            {
                timer1.Enabled = false;
                lblUpdateStatus.Text = "Updates Completed Successfully.";

                prgBarUpdate.Value = 9;
                btnUpdateNow.Enabled = true;
                grpServerSettings.Enabled = true;
            }
        }
    }
}